@@ -63,16 +63,22 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
             mock.patch(
                 SERVERMANAGER + '.ServerPool.rest_call',
                 side_effect=servermanager.RemoteRestError(
-                    reason='Failure to break loop'
+                    reason='Failure to trigger except clause.'
                 )
+            ),
+            mock.patch(
+                SERVERMANAGER + '.LOG.exception',
+                side_effect=KeyError('Failure to break loop')
             )
-        ) as (smock, rmock):
+        ) as (smock, rmock, lmock):
             # should return immediately without consistency capability
             pl.servers._consistency_watchdog()
             self.assertFalse(smock.called)
             pl.servers.capabilities = ['consistency']
-            self.assertRaises(servermanager.RemoteRestError,
+            self.assertRaises(KeyError,
                               pl.servers._consistency_watchdog)
+            rmock.assert_called_with('GET', '/health', '', {}, [], False)
+            self.assertEqual(1, len(lmock.mock_calls))
 
     def test_file_put_contents(self):
         pl = NeutronManager.get_plugin()
@@ -103,6 +109,37 @@ class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):
                 mock.call.write('certdata')
             ])
 
+    def test_capabilities_retrieval(self):
+        sp = servermanager.ServerPool()
+        with mock.patch(HTTPCON) as conmock:
+            rv = conmock.return_value.getresponse.return_value
+            rv.getheader.return_value = 'HASHHEADER'
+
+            # each server will get different capabilities
+            rv.read.side_effect = ['["a","b","c"]', '["b","c","d"]']
+            # pool capabilities is intersection between both
+            self.assertEqual(set(['b', 'c']), sp.get_capabilities())
+            self.assertEqual(2, rv.read.call_count)
+
+            # the pool should cache after the first call so no more
+            # HTTP calls should be made
+            rv.read.side_effect = ['["w","x","y"]', '["x","y","z"]']
+            self.assertEqual(set(['b', 'c']), sp.get_capabilities())
+            self.assertEqual(2, rv.read.call_count)
+
+    def test_capabilities_retrieval_failure(self):
+        sp = servermanager.ServerPool()
+        with mock.patch(HTTPCON) as conmock:
+            rv = conmock.return_value.getresponse.return_value
+            rv.getheader.return_value = 'HASHHEADER'
+            # a failure to parse should result in an empty capability set
+            rv.read.return_value = 'XXXXX'
+            self.assertEqual([], sp.servers[0].get_capabilities())
+
+            # One broken server should affect all capabilities
+            rv.read.side_effect = ['{"a": "b"}', '["b","c","d"]']
+            self.assertEqual(set(), sp.get_capabilities())
+
     def test_reconnect_cached_connection(self):
         sp = servermanager.ServerPool()
         with mock.patch(HTTPCON) as conmock:
