@@ -264,6 +264,8 @@ class TestN1kvNetworkProfiles(N1kvPluginTestCase):
             netp['network_profile']['sub_type'] = 'enhanced' or 'native_vxlan'
             netp['network_profile']['multicast_ip_range'] = ("224.1.1.1-"
                                                              "224.1.1.10")
+        elif segment_type == 'trunk':
+            netp['network_profile']['sub_type'] = 'vlan'
         return netp
 
     def test_create_network_profile_vlan(self):
@@ -278,6 +280,19 @@ class TestN1kvNetworkProfiles(N1kvPluginTestCase):
         res = net_p_req.get_response(self.ext_api)
         self.assertEqual(res.status_int, 201)
 
+    def test_create_network_profile_trunk(self):
+        data = self._prepare_net_profile_data('trunk')
+        net_p_req = self.new_create_request('network_profiles', data)
+        res = net_p_req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, 201)
+
+    def test_create_network_profile_trunk_missing_subtype(self):
+        data = self._prepare_net_profile_data('trunk')
+        data['network_profile'].pop('sub_type')
+        net_p_req = self.new_create_request('network_profiles', data)
+        res = net_p_req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, 400)
+
     def test_create_network_profile_overlay_unreasonable_seg_range(self):
         data = self._prepare_net_profile_data('overlay')
         data['network_profile']['segment_range'] = '10000-100000000001'
@@ -327,6 +342,27 @@ class TestN1kvNetworkProfiles(N1kvPluginTestCase):
         update_res = update_req.get_response(self.ext_api)
         self.assertEqual(update_res.status_int, 400)
 
+    def test_update_network_profiles_with_networks_fail(self):
+        net_p = self._make_test_profile(name='netp1')
+        data = {'network_profile': {'segment_range': '200-210'}}
+        update_req = self.new_update_request('network_profiles',
+                                             data,
+                                             net_p['id'])
+        update_res = update_req.get_response(self.ext_api)
+        self.assertEqual(update_res.status_int, 200)
+        net_data = {'network': {'name': 'net1',
+                                n1kv.PROFILE_ID: net_p['id'],
+                                'tenant_id': 'some_tenant'}}
+        network_req = self.new_create_request('networks', net_data)
+        network_res = network_req.get_response(self.api)
+        self.assertEqual(network_res.status_int, 201)
+        data = {'network_profile': {'segment_range': '300-310'}}
+        update_req = self.new_update_request('network_profiles',
+                                             data,
+                                             net_p['id'])
+        update_res = update_req.get_response(self.ext_api)
+        self.assertEqual(update_res.status_int, 409)
+
     def test_create_overlay_network_profile_invalid_multicast_fail(self):
         net_p_dict = self._prepare_net_profile_data('overlay')
         data = {'network_profile': {'sub_type': 'native_vxlan',
@@ -381,6 +417,81 @@ class TestN1kvNetworkProfiles(N1kvPluginTestCase):
         res = net_p_req.get_response(self.ext_api)
         self.assertEqual(res.status_int, 201)
 
+    def test_update_overlay_network_profile_correct_multicast_pass(self):
+        data = self._prepare_net_profile_data('overlay')
+        net_p_req = self.new_create_request('network_profiles', data)
+        res = net_p_req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, 201)
+        net_p = self.deserialize(self.fmt, res)
+        data = {'network_profile': {'multicast_ip_range':
+                                    '224.0.1.0-224.0.1.100'}}
+        update_req = self.new_update_request('network_profiles',
+                                             data,
+                                             net_p['network_profile']['id'])
+        update_res = update_req.get_response(self.ext_api)
+        self.assertEqual(update_res.status_int, 200)
+
+    def test_create_overlay_network_profile_reservedip_multicast_fail(self):
+        net_p_dict = self._prepare_net_profile_data('overlay')
+        data = {'network_profile': {'multicast_ip_range':
+                                    '224.0.0.100-224.0.1.100'}}
+        net_p_req = self.new_create_request('network_profiles', data,
+                                            net_p_dict)
+        res = net_p_req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, 400)
+
+    def test_update_overlay_network_profile_reservedip_multicast_fail(self):
+        data = self._prepare_net_profile_data('overlay')
+        net_p_req = self.new_create_request('network_profiles', data)
+        res = net_p_req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, 201)
+        net_p = self.deserialize(self.fmt, res)
+        data = {'network_profile': {'multicast_ip_range':
+                                    '224.0.0.11-224.0.0.111'}}
+        update_req = self.new_update_request('network_profiles',
+                                             data,
+                                             net_p['network_profile']['id'])
+        update_res = update_req.get_response(self.ext_api)
+        self.assertEqual(update_res.status_int, 400)
+
+    def test_update_vlan_network_profile_multicast_fail(self):
+        net_p = self._make_test_profile(name='netp1')
+        data = {'network_profile': {'multicast_ip_range':
+                                    '224.0.1.0-224.0.1.100'}}
+        update_req = self.new_update_request('network_profiles',
+                                             data,
+                                             net_p['id'])
+        update_res = update_req.get_response(self.ext_api)
+        self.assertEqual(update_res.status_int, 400)
+
+    def test_update_trunk_network_profile_segment_range_fail(self):
+        data = self._prepare_net_profile_data('trunk')
+        net_p_req = self.new_create_request('network_profiles', data)
+        res = net_p_req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, 201)
+        net_p = self.deserialize(self.fmt, res)
+        data = {'network_profile': {'segment_range':
+                                    '100-200'}}
+        update_req = self.new_update_request('network_profiles',
+                                             data,
+                                             net_p['network_profile']['id'])
+        update_res = update_req.get_response(self.ext_api)
+        self.assertEqual(update_res.status_int, 400)
+
+    def test_update_trunk_network_profile_multicast_fail(self):
+        data = self._prepare_net_profile_data('trunk')
+        net_p_req = self.new_create_request('network_profiles', data)
+        res = net_p_req.get_response(self.ext_api)
+        self.assertEqual(res.status_int, 201)
+        net_p = self.deserialize(self.fmt, res)
+        data = {'network_profile': {'multicast_ip_range':
+                                    '224.0.1.0-224.0.1.100'}}
+        update_req = self.new_update_request('network_profiles',
+                                             data,
+                                             net_p['network_profile']['id'])
+        update_res = update_req.get_response(self.ext_api)
+        self.assertEqual(update_res.status_int, 400)
+
     def test_create_network_profile_populate_vlan_segment_pool(self):
         db_session = db.get_session()
         net_p_dict = self._prepare_net_profile_data('vlan')
@@ -405,6 +516,17 @@ class TestN1kvNetworkProfiles(N1kvPluginTestCase):
                           PHYS_NET,
                           VLAN_MAX + 1)
 
+    def test_delete_network_profile_with_network_fail(self):
+        net_p = self._make_test_profile(name='netp1')
+        net_data = {'network': {'name': 'net1',
+                                n1kv.PROFILE_ID: net_p['id'],
+                                'tenant_id': 'some_tenant'}}
+        network_req = self.new_create_request('networks', net_data)
+        network_res = network_req.get_response(self.api)
+        self.assertEqual(network_res.status_int, 201)
+        self._delete('network_profiles', net_p['id'],
+                     expected_code=409)
+
     def test_delete_network_profile_deallocate_vlan_segment_pool(self):
         db_session = db.get_session()
         net_p_dict = self._prepare_net_profile_data('vlan')
