@@ -1075,6 +1075,7 @@ class FakeV4Network:
     id = 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'
     subnets = [FakeV4Subnet()]
     ports = [FakePort1()]
+    namespace = 'qdhcp-aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'
 
 
 class FakeV4NetworkNoSubnet:
@@ -1165,7 +1166,7 @@ class TestDeviceManager(base.BaseTestCase):
                                            namespace=net.namespace))
         self.mock_driver.assert_has_calls(expected)
 
-        dh._set_default_route.assert_called_once_with(net, port)
+        dh._set_default_route.assert_called_once_with(net, 'tap12345678-12')
 
     def test_setup(self):
         cfg.CONF.set_override('enable_metadata_network', False)
@@ -1308,35 +1309,26 @@ class TestDeviceManager(base.BaseTestCase):
                 self.assertEqual(dh.get_device_id(fake_net), expected)
                 uuid5.assert_called_once_with(uuid.NAMESPACE_DNS, 'localhost')
 
-    def _get_device_manager_with_mock_device(self, conf, device):
-            dh = dhcp.DeviceManager(conf, cfg.CONF.root_helper, None)
-            dh._get_device = mock.Mock(return_value=device)
-            return dh
-
     def test_update(self):
         # Try with namespaces and no metadata network
         cfg.CONF.set_override('use_namespaces', True)
         cfg.CONF.set_override('enable_metadata_network', False)
         dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
         dh._set_default_route = mock.Mock()
-
         network = mock.Mock()
-        port = mock.Mock()
-        dh.plugin = mock.Mock()
-        dh.plugin.get_dhcp_port.return_value = port
-        dh.update(network)
 
-        dh._set_default_route.assert_called_once_with(network, port)
+        dh.update(network, 'ns-12345678-12')
+
+        dh._set_default_route.assert_called_once_with(network,
+                                                      'ns-12345678-12')
 
         # No namespaces, shouldn't set default route.
         cfg.CONF.set_override('use_namespaces', False)
         cfg.CONF.set_override('enable_metadata_network', False)
         dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
         dh._set_default_route = mock.Mock()
-        dh.plugin = mock.Mock()
-        dh.plugin.get_dhcp_port.return_value = port
 
-        dh.update(FakeV4Network())
+        dh.update(FakeV4Network(), 'tap12345678-12')
 
         self.assertFalse(dh._set_default_route.called)
 
@@ -1345,10 +1337,8 @@ class TestDeviceManager(base.BaseTestCase):
         cfg.CONF.set_override('enable_metadata_network', True)
         dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
         dh._set_default_route = mock.Mock()
-        dh.plugin = mock.Mock()
-        dh.plugin.get_dhcp_port.return_value = port
 
-        dh.update(FakeV4Network())
+        dh.update(FakeV4Network(), 'ns-12345678-12')
 
         self.assertTrue(dh._set_default_route.called)
 
@@ -1358,123 +1348,109 @@ class TestDeviceManager(base.BaseTestCase):
         dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
         dh._set_default_route = mock.Mock()
 
-        dh.update(FakeV4Network())
+        dh.update(FakeV4Network(), 'ns-12345678-12')
 
         self.assertFalse(dh._set_default_route.called)
 
     def test_set_default_route(self):
-        device = mock.Mock()
-        device.route.get_gateway.return_value = None
-
-        # Basic one subnet with gateway.
-        dh = self._get_device_manager_with_mock_device(cfg.CONF, device)
-        network = FakeV4Network()
-
-        dh._set_default_route(network, mock.Mock())
+        dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
+        with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
+            device = mock.Mock()
+            mock_IPDevice.return_value = device
+            device.route.get_gateway.return_value = None
+            # Basic one subnet with gateway.
+            network = FakeV4Network()
+            dh._set_default_route(network, 'tap-name')
 
         device.route.get_gateway.assert_called_once()
         self.assertFalse(device.route.delete_gateway.called)
         device.route.add_gateway.assert_called_once_with('192.168.0.1')
 
     def test_set_default_route_no_subnet(self):
-        device = mock.Mock()
-        device.route.get_gateway.return_value = None
-
-        # Try a namespace but no subnet.
-        dh = self._get_device_manager_with_mock_device(cfg.CONF, device)
-        network = FakeV4NetworkNoSubnet()
-
-        dh._set_default_route(network, mock.Mock())
+        dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
+        with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
+            device = mock.Mock()
+            mock_IPDevice.return_value = device
+            device.route.get_gateway.return_value = None
+            network = FakeV4NetworkNoSubnet()
+            network.namespace = 'qdhcp-1234'
+            dh._set_default_route(network, 'tap-name')
 
         device.route.get_gateway.assert_called_once()
         self.assertFalse(device.route.delete_gateway.called)
         self.assertFalse(device.route.add_gateway.called)
 
     def test_set_default_route_no_subnet_delete_gateway(self):
-        device = mock.Mock()
-        device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
-
-        # Try a namespace but no subnet where a gateway needs to be deleted.
-        dh = self._get_device_manager_with_mock_device(cfg.CONF, device)
-        network = FakeV4NetworkNoSubnet()
-
-        dh._set_default_route(network, mock.Mock())
+        dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
+        with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
+            device = mock.Mock()
+            mock_IPDevice.return_value = device
+            device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
+            network = FakeV4NetworkNoSubnet()
+            network.namespace = 'qdhcp-1234'
+            dh._set_default_route(network, 'tap-name')
 
         device.route.get_gateway.assert_called_once()
         device.route.delete_gateway.assert_called_once_with('192.168.0.1')
         self.assertFalse(device.route.add_gateway.called)
 
     def test_set_default_route_no_gateway(self):
-        device = mock.Mock()
-        device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
-
-        # Try a subnet with no gateway
-        dh = self._get_device_manager_with_mock_device(cfg.CONF, device)
-        network = FakeV4NetworkNoGateway()
-
-        dh._set_default_route(network, mock.Mock())
+        dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
+        with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
+            device = mock.Mock()
+            mock_IPDevice.return_value = device
+            device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
+            network = FakeV4NetworkNoGateway()
+            network.namespace = 'qdhcp-1234'
+            dh._set_default_route(network, 'tap-name')
 
         device.route.get_gateway.assert_called_once()
         device.route.delete_gateway.assert_called_once_with('192.168.0.1')
         self.assertFalse(device.route.add_gateway.called)
 
     def test_set_default_route_do_nothing(self):
-        device = mock.Mock()
-        device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
-
-        # Try a subnet where the gateway doesn't change.  Should do nothing.
-        dh = self._get_device_manager_with_mock_device(cfg.CONF, device)
-        network = FakeV4Network()
-
-        dh._set_default_route(network, mock.Mock())
+        dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
+        with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
+            device = mock.Mock()
+            mock_IPDevice.return_value = device
+            device.route.get_gateway.return_value = dict(gateway='192.168.0.1')
+            network = FakeV4Network()
+            dh._set_default_route(network, 'tap-name')
 
         device.route.get_gateway.assert_called_once()
         self.assertFalse(device.route.delete_gateway.called)
         self.assertFalse(device.route.add_gateway.called)
 
     def test_set_default_route_change_gateway(self):
-        device = mock.Mock()
-        device.route.get_gateway.return_value = dict(gateway='192.168.0.2')
-
-        # Try a subnet with a gateway this is different than the current.
-        dh = self._get_device_manager_with_mock_device(cfg.CONF, device)
-        network = FakeV4Network()
-
-        dh._set_default_route(network, mock.Mock())
+        dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
+        with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
+            device = mock.Mock()
+            mock_IPDevice.return_value = device
+            device.route.get_gateway.return_value = dict(gateway='192.168.0.2')
+            network = FakeV4Network()
+            dh._set_default_route(network, 'tap-name')
 
         device.route.get_gateway.assert_called_once()
         self.assertFalse(device.route.delete_gateway.called)
         device.route.add_gateway.assert_called_once_with('192.168.0.1')
 
     def test_set_default_route_two_subnets(self):
-        device = mock.Mock()
-        device.route.get_gateway.return_value = None
-
-        # Try two subnets.  Should set gateway from the first.
-        dh = self._get_device_manager_with_mock_device(cfg.CONF, device)
-        network = FakeV4Network()
-        subnet2 = FakeV4Subnet()
-        subnet2.gateway_ip = '192.168.1.1'
-        network.subnets = [subnet2, FakeV4Subnet()]
-
-        dh._set_default_route(network, mock.Mock())
+        # Try two subnets. Should set gateway from the first.
+        dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
+        with mock.patch.object(dhcp.ip_lib, 'IPDevice') as mock_IPDevice:
+            device = mock.Mock()
+            mock_IPDevice.return_value = device
+            device.route.get_gateway.return_value = None
+            network = FakeV4Network()
+            subnet2 = FakeV4Subnet()
+            subnet2.gateway_ip = '192.168.1.1'
+            network.subnets = [subnet2, FakeV4Subnet()]
+            dh._set_default_route(network, 'tap-name')
 
         device.route.get_gateway.assert_called_once()
         self.assertFalse(device.route.delete_gateway.called)
         device.route.add_gateway.assert_called_once_with('192.168.1.1')
 
-    def test_get_device_dont_call_get_dhcp_port(self):
-        dh = dhcp.DeviceManager(cfg.CONF, cfg.CONF.root_helper, None)
-        dh.get_interface_name = mock.Mock()
-        dh.plugin = mock.Mock()
-        network = mock.Mock()
-        port = mock.Mock()
-
-        dh._get_device(network, port)
-
-        self.assertFalse(dh.plugin.get_dhcp_port.called)
-        dh.get_interface_name.assert_called_once_with(network, port)
-
 
 class TestDictModel(base.BaseTestCase):
     def test_basic_dict(self):
