@@ -15,6 +15,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import contextlib
 import os
 
 import mock
@@ -1016,26 +1017,29 @@ tag:tag1,249,%s,%s""".lstrip() % (fake_v6,
 
         exp_args = ['kill', '-HUP', 5]
 
-        with mock.patch('os.path.isdir') as isdir:
-            isdir.return_value = True
-            with mock.patch.object(dhcp.Dnsmasq, 'active') as active:
-                active.__get__ = mock.Mock(return_value=True)
-                with mock.patch.object(dhcp.Dnsmasq, 'pid') as pid:
-                    pid.__get__ = mock.Mock(return_value=5)
-                    dm = dhcp.Dnsmasq(self.conf, FakeDualNetwork(),
-                                      version=float(2.59))
+        fake_net = FakeDualNetwork()
+        dm = dhcp.Dnsmasq(self.conf, fake_net, version=float(2.59))
 
-                    method_name = '_make_subnet_interface_ip_map'
-                    with mock.patch.object(dhcp.Dnsmasq,
-                                           method_name) as ip_map:
-                        ip_map.return_value = {}
-                        dm.reload_allocations()
-                        self.assertTrue(ip_map.called)
+        with contextlib.nested(
+            mock.patch('os.path.isdir', return_value=True),
+            mock.patch.object(dhcp.Dnsmasq, 'active'),
+            mock.patch.object(dhcp.Dnsmasq, 'pid'),
+            mock.patch.object(dhcp.Dnsmasq, 'interface_name'),
+            mock.patch.object(dhcp.Dnsmasq, '_make_subnet_interface_ip_map'),
+            mock.patch.object(dm, 'device_manager')
+        ) as (isdir, active, pid, interface_name, ip_map, device_manager):
+            active.__get__ = mock.Mock(return_value=True)
+            pid.__get__ = mock.Mock(return_value=5)
+            interface_name.__get__ = mock.Mock(return_value='tap12345678-12')
+            ip_map.return_value = {}
+            dm.reload_allocations()
 
+        self.assertTrue(ip_map.called)
         self.safe.assert_has_calls([mock.call(exp_host_name, exp_host_data),
                                     mock.call(exp_addn_name, exp_addn_data),
                                     mock.call(exp_opt_name, exp_opt_data)])
         self.execute.assert_called_once_with(exp_args, 'sudo')
+        device_manager.update.assert_called_with(fake_net, 'tap12345678-12')
 
     def test_reload_allocations_stale_pid(self):
         (exp_host_name, exp_host_data,
@@ -1057,8 +1061,9 @@ tag:tag1,249,%s,%s""".lstrip() % (fake_v6,
                     method_name = '_make_subnet_interface_ip_map'
                     with mock.patch.object(dhcp.Dnsmasq, method_name) as ipmap:
                         ipmap.return_value = {}
-                        dm.reload_allocations()
-                        self.assertTrue(ipmap.called)
+                        with mock.patch.object(dhcp.Dnsmasq, 'interface_name'):
+                            dm.reload_allocations()
+                            self.assertTrue(ipmap.called)
 
             self.safe.assert_has_calls([
                 mock.call(exp_host_name, exp_host_data),
